"""Exercises for itertools"""


def lstrip():
    """Return iterable with strip_value items removed from beginning."""


def total_length():
    """Return the total number of items in all given iterables."""


def compact():
    """Lazily return elements of iterable, removing adjacent duplicates."""


def stop_on():
    """Yield from the iterable until the given value is reached."""


def random_number_generator():
    """Return a generator that yields 4 forever."""


def running_mean():
    """Yield the running mean while looping over the iterable."""
